
# q-expansion of newform 5400.2.a.g, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5400
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [-3], [4], [6], [6], [-1], [-6], [-10], [1], [11], [0], [-3], [8], [12], [-6], [5], [-4], [-2], [-9], [-5], [18], [2], [-5], [0], [-9], [2], [-9], [12], [-16], [14], [8], [19], [-2], [-11], [3], [20], [-2], [-10], [24], [26], [12], [-7], [8], [0], [-12], [19], [-4], [-25], [26], [-12], [10], [14], [28], [-18], [6], [15], [5], [-8], [19], [22], [-5], [-8], [-22], [26], [-13], [22], [6], [17], [8], [-18], [-28], [-1], [16], [30], [-36], [19], [-16], [26], [18], [-13], [22], [21], [1], [-22], [-6], [-2], [-2], [-29], [-22], [-10], [-16], [12], [25], [26], [8], [12], [1], [-21], [-27], [18], [-32], [-42], [-33], [-23], [38], [-30], [40], [35], [-25], [-25], [-4], [-37], [-28], [30], [-4], [-48], [-16], [-34], [-3], [29], [-4], [46], [-20], [-6], [-37], [42], [47], [2], [-28], [-16], [-37], [29], [16], [-18], [6], [-43], [-44], [30], [5], [-32], [0], [0], [-5], [14], [-26], [44], [43], [-34], [-13], [22], [-37], [-24], [-25], [-54], [-25], [-30], [-39], [32], [-6], [4], [9], [-48], [-14], [-28], [-5], [-6], [49], [-30], [26], [-35], [0], [-53], [24], [10], [49], [-10], [16], [1], [40], [16], [29], [18], [-2], [-24], [2], [40], [53], [-2], [35], [-54], [-51], [8], [60], [10], [-1], [-59], [-4], [-24], [-4], [23], [17], [-15], [12], [14], [-11], [-42], [42], [40], [14], [30], [52], [-20], [-6], [-9], [31], [-50], [24], [-24], [-1], [31], [2], [20], [-28], [-54], [12], [18], [32], [24], [49], [32], [23], [-64], [-35], [36], [5], [68], [28], [50], [44], [28], [43], [-10], [14], [-42], [35], [0], [43], [-46], [30], [-42], [-72], [70], [-4], [24], [-1], [28], [-50], [-70], [53], [72], [-41], [-21], [0], [4], [-66], [30], [-44], [-54], [-15], [29], [-34], [52], [-37], [-83], [18], [-19], [-53], [30], [-24], [-4], [18], [-70], [-28], [2], [-66], [42], [-63], [-38], [-26], [-38], [-60], [-30], [-5], [48], [-65], [66], [18], [-1], [17], [36], [33], [18], [55], [-61], [-28], [-49], [-18], [31], [-12], [2], [74], [23], [32], [-75], [-42], [34], [47], [26], [-3], [18], [20], [-29], [-60], [-59], [29], [-73], [-70], [-30], [26], [72], [32], [42], [13], [36], [-55], [-24], [-3], [-24], [19], [-2], [-84], [-15], [66], [-50], [-73], [-45], [-84], [-68], [-45], [-5], [-76], [25], [10], [-70], [-48], [0], [-70], [64], [-73], [48], [36], [-16], [97], [45], [34], [11], [30], [98], [65], [32], [46], [-23], [-71], [-54], [6], [-38], [-84], [-17], [-70], [46], [-83], [-32], [-19], [22], [65], [13], [-35], [-74], [-67], [4], [-14], [-52], [-20], [11], [40], [24], [-45], [66], [-74], [32], [101], [42], [-90], [32], [41], [-12], [-68], [70], [89], [10], [-16], [16], [-9], [70], [54], [83], [10], [4], [-100], [3], [28], [-66], [26], [-54], [52], [38], [11], [76]]
